"use strict";

{
	const C3 = self.C3;
	C3.Behaviors.skymenBoids.Instance = class skymenBoidsInstance extends C3.SDKBehaviorInstanceBase
	{
		constructor(behInst, properties)
		{
			super(behInst);
			this.inst = this._inst;
		    // Define additional properties required for autonomous steering calculations
		    // Velocity
		    this.dx = 0;
			this.dy = 0;
		    // Limits
		    this.maxForce = 0.1;
		    this.maxSpeed = 5;
			this.setAngle = true;
		    // Steering force
		    this.sx = 0;
		    this.sy = 0;

		    this.enabled = true;
			if (properties)
			{
				this.maxSpeed = properties[0];
				this.maxForce = properties[1];
				this.setAngle = properties[2];
			}
			
			// Opt-in to getting calls to Tick()
			this._StartTicking();
		}

		Release()
		{
			super.Release();
		}
		
		GetDebuggerProperties()
		{
			return [{
				title: "Boids",
				properties: [
					{name: "$Max Force",	value: this.maxForce,	onedit: (val) => { this.maxForce = parseFloat(val) }},
					{name: "$Max Speed",	value: this.maxSpeed,	onedit: (val) => { this.maxSpeed = parseFloat(val) }},
					{name: "$Set Angle",	value: this.setAngle,	onedit: (val) => { this.setAngle = !!val }},
					{name: "$dx",			value: this.dx},
					{name: "$dy",			value: this.dy},
					{name: "$sx",			value: this.sx},
					{name: "$sy",			value: this.sy},
				]
			}];
		}
		
		SaveToJson()
		{
			let keys = ["sx", "sy", "maxForce", "dx", "dy", "setAngle", "maxSpeed"]
			let o = {};
			keys.forEach(key => {
				o[key] = this[key];
			});
			return {
				...o,
			};
		}

		LoadFromJson(o)
		{
			let keys = ["sx", "sy", "maxForce", "dx", "dy", "setAngle", "maxSpeed"]
			keys.forEach(key => {
				this[key] = o[key];
			});
		}
		
		
		Tick()
		{
			const dt = this._runtime.GetDt(this._inst);
			const timescale = this.GetRuntime().GetTimeScale();
			const wi = this._inst.GetWorldInfo();
		    // Calculate magnitude of steering force
		    var force = Math.sqrt(this.sx * this.sx + this.sy * this.sy);

		    // Truncate if greater than maxForce
		    var scaleFactor = (force > this.maxForce) ? this.maxForce / force : 1;
		    this.sx = this.sx * scaleFactor * dt;
		    this.sy = this.sy * scaleFactor * dt;

		    // Accelerate
		    this.dx += this.sx;
		    this.dy += this.sy;

		    // Calculate magnitude of current speed
		    var speed = Math.sqrt(this.dx * this.dx + this.dy * this.dy);

		    // Truncate speed if over max speed
		    if(speed > this.maxSpeed) {
		      var scaleFactor = this.maxSpeed / speed;
		      this.dx = this.dx * scaleFactor;
		      this.dy = this.dy * scaleFactor;
		    }
			var mx = this.dx * dt;
			var my = this.dy * dt;

		    // Update position
		    wi.OffsetXY(mx, my);

		    // Update angle
		    if (this.setAngle) wi.SetAngle(this.getAngle());

		    // IMPORTANT - always call after updating position or angle
		    wi.SetBboxChanged();

		    // Reset the steering force for the next tick
		    this.sx = 0;
		    this.sy = 0;
		}

	  	getSpeed()
		{
			return Math.sqrt(this.dx * this.dx + this.dy * this.dy);
		}
		
		getAngle()
		{
			return Math.atan2(this.dy, this.dx);
		}

		
	};
}